@@ -20,7 +20,7 @@
 #include "config.h"
 #endif
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "gul-glade.h"
 #include "galeon-marshal.h"
 #include "bookmarks-editor-dockable.h"
@@ -31,6 +31,7 @@
 #include "gul-state.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
+#include "galeon-debug.h"
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkscrolledwindow.h>
@@ -45,10 +46,6 @@
 #include <bonobo/bonobo-ui-component.h>
 #include <bonobo/bonobo-file-selector-util.h>
 
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
-
 /**
  * Private data
  */
@@ -98,7 +95,7 @@ gb_editor_dockable_finalize_impl (GObject *o)
 	
 	g_free (p);
 	
-	DEBUG_MSG (("GbEditorDockable finalized\n"));
+	LOG ("GbEditorDockable finalized");
 	
 	G_OBJECT_CLASS (gb_editor_class)->finalize (o);
 }
