@@ -27,6 +27,8 @@
 #include "gul-gui.h"
 #include "bookmarks-widgets-private.h"
 #include "bookmarks-single-editor.h"
+#include "bookmarks-dnd.h"
+#include "galeon-dnd.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
 
@@ -34,11 +36,14 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkimage.h>
-#include <gtk/gtkcombo.h>
+#include <gtk/gtkcomboboxentry.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenuitem.h>
-#include <libgnome/gnome-i18n.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkeditable.h>
+
+#include <glib/gi18n.h>
 
 /**
  * Private data
@@ -58,6 +63,18 @@ struct _GbSmartSiteTbWidgetPrivate {
 
 static GSList *all_instances = NULL;
 
+const GtkTargetEntry text_drop_types[] =
+{
+	/* Drop targets copied from gtkentry.c */
+	{ "UTF8_STRING",   0, 0 },
+	{ "COMPOUND_TEXT", 0, 0 },
+	{ "TEXT",	   0, 0 },
+	{ "STRING",	   0, 0 },
+	/* And the one that mozilla support */
+	{ GALEON_DND_TEXT_TYPE, 0, GALEON_DND_TEXT },
+};
+
+
 /**
  * Private functions, only availble from this file
  */
@@ -82,6 +99,16 @@ static void		gb_smart_site_tb_widget_conf_changed_cb	(GConfClient* client,
 								 guint cnxn_id,
 								 GConfEntry *entry,
 								 gpointer user_data);
+static gboolean         gb_smart_site_tb_widget_button_press_cb (GtkWidget *wid, 
+                                                                 GdkEventButton *event, 
+                                                                 GbSmartSiteTbWidget *w);
+static void		gb_smart_site_tb_widget_drag_data_received_cb (GtkWidget *widget,
+								       GdkDragContext *dc,
+								       gint x, gint y,
+								       GtkSelectionData *selection_data,
+								       guint info,
+								       guint t,
+								       GbSmartSiteTbWidget *w);
 
 
 static gpointer gb_tb_widget_class;
@@ -103,6 +130,8 @@ gb_smart_site_tb_widget_class_init (GbSmartSiteTbWidgetClass *klass)
 
 	eel_gconf_notification_add 
 		(CONF_BOOKMARKS_HISTORY, gb_smart_site_tb_widget_conf_changed_cb, NULL);
+	eel_gconf_notification_add 
+		(CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS, gb_smart_site_tb_widget_conf_changed_cb, NULL);
 }
 
 static void 
@@ -122,6 +151,10 @@ gb_smart_site_tb_widget_finalize_impl (GObject *o)
 	
 	all_instances = g_slist_remove (all_instances, w);
 	
+	g_free (p->entries);
+	g_free (p->combos);
+	g_free (p->entries_sizes);
+
 	g_free (p);
 	
 	G_OBJECT_CLASS (gb_tb_widget_class)->finalize (o);
@@ -156,9 +189,8 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 	GbSmartSiteTbWidgetPrivate *p = w->priv;
 	GbSmartSite *ss = GB_SMART_SITE (gb_tb_widget_get_bookmark (GB_TB_WIDGET (w)));
 	int num_entries = gb_smart_site_get_num_fields (ss);
+	gboolean hide_arrows = eel_gconf_get_boolean (CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS);
 	int i;
-	GtkWidget *fim;
-	GtkWidget *ufim;
 
 	p->num_entries = num_entries;
 
@@ -180,18 +212,23 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 	gtk_table_attach_defaults (GTK_TABLE (p->mainwid), p->smart_box, 1, 2, 0, 1);
 	gtk_widget_show (p->smart_box);
 
-	p->unfold_button = gtk_button_new ();
-	p->fold_button = gtk_button_new ();
-	gtk_box_pack_start (GTK_BOX (p->smart_box), p->unfold_button, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (p->smart_box), p->fold_button, TRUE, TRUE, 0);
-	fim = gtk_image_new_from_pixbuf (smart_bm_fold_pixbuf_data);
-	ufim = gtk_image_new_from_pixbuf (smart_bm_unfold_pixbuf_data);
-	gtk_container_add (GTK_CONTAINER (p->unfold_button), ufim);
-	gtk_container_add (GTK_CONTAINER (p->fold_button), fim);
-	gtk_button_set_relief (GTK_BUTTON (p->fold_button), GTK_RELIEF_NONE);
-	GTK_WIDGET_SET_FLAGS (GTK_BUTTON (p->fold_button), GTK_CAN_FOCUS);
-	gtk_button_set_relief (GTK_BUTTON (p->unfold_button), GTK_RELIEF_NONE);
-	GTK_WIDGET_SET_FLAGS (GTK_BUTTON (p->unfold_button), GTK_CAN_FOCUS);
+	if (!eel_gconf_get_boolean (CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS))
+	{
+		GtkWidget *fim;
+		GtkWidget *ufim;
+		p->unfold_button = gtk_button_new ();
+		p->fold_button = gtk_button_new ();
+		gtk_box_pack_start (GTK_BOX (p->smart_box), p->unfold_button, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (p->smart_box), p->fold_button, TRUE, TRUE, 0);
+		fim = gtk_image_new_from_pixbuf (smart_bm_fold_pixbuf_data);
+		ufim = gtk_image_new_from_pixbuf (smart_bm_unfold_pixbuf_data);
+		gtk_container_add (GTK_CONTAINER (p->unfold_button), ufim);
+		gtk_container_add (GTK_CONTAINER (p->fold_button), fim);
+		gtk_button_set_relief (GTK_BUTTON (p->fold_button), GTK_RELIEF_NONE);
+		GTK_WIDGET_SET_FLAGS (GTK_BUTTON (p->fold_button), GTK_CAN_FOCUS);
+		gtk_button_set_relief (GTK_BUTTON (p->unfold_button), GTK_RELIEF_NONE);
+		GTK_WIDGET_SET_FLAGS (GTK_BUTTON (p->unfold_button), GTK_CAN_FOCUS);
+	}
 
 	p->entries = g_new0 (GtkWidget *, num_entries + 1);
 	p->combos = g_new0 (GtkWidget *, num_entries + 1);
@@ -201,11 +238,18 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 		if (eel_gconf_get_boolean (CONF_BOOKMARKS_HISTORY))
 		{
 			GSList *history = gb_smart_site_get_history (ss, i);
-			GList *history_l = gslist_to_glist (history);
+			GList *l, *history_l = gslist_to_glist (history);
+
 			history_l = g_list_prepend (history_l, "");
-			p->combos[i] = gtk_combo_new ();
-			p->entries[i] = GTK_COMBO (p->combos[i])->entry;
-			gtk_combo_set_popdown_strings (GTK_COMBO (p->combos[i]), history_l);
+			p->combos[i] = gtk_combo_box_entry_new_text();
+			p->entries[i] = GTK_BIN(p->combos[i])->child;
+
+			for (l = history_l; l; l = l->next)
+			{
+				gtk_combo_box_append_text(GTK_COMBO_BOX(p->combos[i]),
+				                          (const gchar *)l->data);
+			}
+
 			g_slist_foreach (history, (GFunc) g_free, NULL);
 			g_slist_free (history);
 			g_list_free (history_l);
@@ -217,7 +261,7 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 		}
 
 		p->entries_sizes[i] = gb_smart_site_get_entry_size (ss, i);
-		if (!ss->folded)
+		if (!ss->folded || hide_arrows)
 		{
 			gtk_widget_show (p->combos[i] ? p->combos[i] : p->entries[i]);
 		}
@@ -230,17 +274,22 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 	p->entries[num_entries] = NULL;
 	p->combos[num_entries] = NULL;
 
-	if (ss->folded)
+	if (!hide_arrows)
 	{
-		gtk_widget_show_all (p->unfold_button);
-	}
-	else
-	{
-		gtk_widget_show_all (p->fold_button);
+
+		if (ss->folded)
+		{
+			gtk_widget_show_all (p->unfold_button);
+		}
+		else
+		{
+			gtk_widget_show_all (p->fold_button);
+		}
+
+		g_signal_connect (p->fold_button, "clicked", G_CALLBACK (gb_smart_site_tb_widget_fold_cb), w);
+		g_signal_connect (p->unfold_button, "clicked", G_CALLBACK (gb_smart_site_tb_widget_unfold_cb), w);
 	}
 
-	g_signal_connect (p->fold_button, "clicked", G_CALLBACK (gb_smart_site_tb_widget_fold_cb), w);
-	g_signal_connect (p->unfold_button, "clicked", G_CALLBACK (gb_smart_site_tb_widget_unfold_cb), w);
 	g_signal_connect (ss, "visibility-changed", 
 			  G_CALLBACK (gb_smart_site_tb_widget_visibility_changed_cb), w);
 	g_signal_connect (ss, "history-changed", 
@@ -271,6 +320,8 @@ gb_smart_site_tb_widget_rebuild_impl (GbTbWidget *w)
 	p->main_button = gtk_button_new ();
 	g_signal_connect (p->main_button, "clicked", 
 			  G_CALLBACK (gb_smart_site_tb_widget_clicked_cb), w);
+	g_signal_connect (p->main_button, "button_press_event",
+			  G_CALLBACK (gb_smart_site_tb_widget_button_press_cb), w);
 
 	gtk_button_set_relief (GTK_BUTTON (p->main_button), GTK_RELIEF_NONE);
 	GTK_WIDGET_SET_FLAGS (GTK_BUTTON (p->main_button), GTK_CAN_FOCUS);
@@ -279,6 +330,17 @@ gb_smart_site_tb_widget_rebuild_impl (GbTbWidget *w)
 	gb_widgets_fill_tb_item (GTK_CONTAINER (p->main_button), GB_BOOKMARK (site), GB_SITE (site)->url);
 
 	gb_tb_widget_setup_context_menu (w, p->main_button);
+	gb_bookmark_dnd_drag_source_set (GTK_BIN (p->main_button)->child, GB_BOOKMARK (site));
+
+	if (gb_smart_site_get_num_fields (site) == 1 )
+	{
+		gtk_drag_dest_set (p->main_button, GTK_DEST_DEFAULT_ALL,
+				   text_drop_types, G_N_ELEMENTS (text_drop_types),
+				   GDK_ACTION_COPY);
+
+		g_signal_connect (p->main_button, "drag_data_received",
+				  G_CALLBACK (gb_smart_site_tb_widget_drag_data_received_cb), w);
+	}
 
 	if (!p->smart_box)
 	{
@@ -298,6 +360,8 @@ gb_smart_site_tb_widget_unfold_cb (GtkWidget *button, GbSmartSiteTbWidget *w)
 {
 	GbSmartSite *ss = GB_SMART_SITE (gb_tb_widget_get_bookmark (GB_TB_WIDGET (w)));
 	gb_smart_site_set_folded (ss, FALSE);
+	/* bug 89338 */
+	gtk_widget_grab_focus (w->priv->entries[0]);
 }
 
 static void
@@ -305,13 +369,14 @@ gb_smart_site_tb_widget_visibility_changed_cb (GbSmartSite *ss, GbSmartSiteTbWid
 {
 	GbSmartSiteTbWidgetPrivate *p = w->priv;
 	gint i;
+	gboolean hide_arrows = eel_gconf_get_boolean (CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS);
 
 	g_return_if_fail (gb_tb_widget_get_bookmark (GB_TB_WIDGET (w)) == GB_BOOKMARK (ss));
 	g_return_if_fail (p->entries && p->combos);
 
 	for (i = 0; p->entries[i]; i++)
 	{
-		if (!ss->folded)
+		if (!ss->folded || hide_arrows)
 		{
 			gtk_widget_show (p->combos[i] ? p->combos[i] : p->entries[i]);
 		}
@@ -320,20 +385,30 @@ gb_smart_site_tb_widget_visibility_changed_cb (GbSmartSite *ss, GbSmartSiteTbWid
 			gtk_widget_hide (p->combos[i] ? p->combos[i] : p->entries[i]);
 		}
 	}
-	if (ss->folded)
+	
+	if (!hide_arrows)
 	{
-		gtk_widget_hide (p->fold_button);
-		gtk_widget_show_all (p->unfold_button);
-	}
+		if (ss->folded)
+		{
+			gtk_widget_hide (p->fold_button);
+			gtk_widget_show_all (p->unfold_button);
+		}
+		else
+		{
+			gtk_widget_hide (p->unfold_button);
+			gtk_widget_show_all (p->fold_button);
+		}
+	} 
 	else
 	{
+		gtk_widget_hide (p->fold_button);
 		gtk_widget_hide (p->unfold_button);
-		gtk_widget_show_all (p->fold_button);
 	}
 }
 
 static void
-gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w)
+gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w, 
+					     GbBookmarkActivatedFlags flags)
 {
 	GbSmartSiteTbWidgetPrivate *p = w->priv;
 	gchar *url;
@@ -353,7 +428,7 @@ gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w)
 	args[num_args] = NULL;
 
 	url = gb_smart_site_subst_args (ss, args);
-	gb_activated_activate (w, GB_BOOKMARK (ss), url, GB_BAF_DEFAULT);
+	gb_activated_activate (w, GB_BOOKMARK (ss), url, flags);
 	g_free (url);
 
 	for (i = 0; i < num_args; i++)
@@ -363,21 +438,6 @@ gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w)
 	g_free (args);
 }
 
-static gboolean
-gb_smart_site_tb_widget_key_press_event_cb (GtkWidget *entry, GdkEventKey *event, GbSmartSiteTbWidget *w)
-{
-	
-	if (!(event->keyval == GDK_Return || event->keyval == GDK_KP_Enter))
-	{
-		return FALSE;
-	}
-	else
-	{
-		gb_smart_site_tb_widget_do_the_smart_thing (w);
-		return TRUE;
-	}
-}
-
 static gboolean 
 gb_smart_site_tb_widget_entries_empty (GbSmartSiteTbWidget *w)
 {
@@ -401,6 +461,33 @@ gb_smart_site_tb_widget_entries_empty (GbSmartSiteTbWidget *w)
 	return ret;
 }
 
+static gboolean
+gb_smart_site_tb_widget_key_press_event_cb (GtkWidget *entry, GdkEventKey *event, GbSmartSiteTbWidget *w)
+{
+	GbBookmarkActivatedFlags flags;
+
+	if (!(event->keyval == GDK_Return || event->keyval == GDK_KP_Enter))
+	{
+		return FALSE;
+	}
+
+	flags = GB_BAF_DEFAULT;
+	if (event->state & GDK_CONTROL_MASK)
+		flags = GB_BAF_NEW_TAB_OR_WINDOW;
+
+	if (gb_smart_site_tb_widget_entries_empty (w))
+	{
+		gb_smart_site_tb_widget_do_the_smart_thing (w, flags);
+	}
+	else
+	{
+		GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
+		gb_activated_activate (w, b, GB_SITE (b)->url, flags);
+	}
+
+	return TRUE;
+}
+
 static void
 gb_smart_site_tb_widget_clicked_cb (GtkButton *button, GbSmartSiteTbWidget *w)
 {
@@ -409,7 +496,7 @@ gb_smart_site_tb_widget_clicked_cb (GtkButton *button, GbSmartSiteTbWidget *w)
 	{
 		if (gb_smart_site_tb_widget_entries_empty (w))
 		{
-			gb_smart_site_tb_widget_do_the_smart_thing (w);
+			gb_smart_site_tb_widget_do_the_smart_thing (w, GB_BAF_DEFAULT);
 		}
 		else
 		{
@@ -428,6 +515,29 @@ gb_smart_site_tb_widget_entry_width_changed_cb (GbSmartSite *b, gint index, guin
 	gtk_widget_set_size_request (p->entries[index], p->entries_sizes[index], -1);
 }
 
+
+static gboolean
+gb_smart_site_tb_widget_button_press_cb (GtkWidget *wid, 
+					 GdkEventButton *event, 
+					 GbSmartSiteTbWidget *w)
+{
+	GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
+	if (b && event->button == 2)
+	{
+		if (gb_smart_site_tb_widget_entries_empty (w))
+		{
+			gb_smart_site_tb_widget_do_the_smart_thing (w, GB_BAF_NEW_TAB_OR_WINDOW);
+		}
+		else
+		{
+			gb_activated_event (w, b, GB_SITE (b)->url, 
+					    GB_BAF_NEW_TAB_OR_WINDOW, (GdkEvent *) event);
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static void
 gb_smart_site_tb_widget_history_changed_cb (GbSmartSite *ss, GbSmartSiteTbWidget *w)
 {
@@ -439,11 +549,22 @@ gb_smart_site_tb_widget_history_changed_cb (GbSmartSite *ss, GbSmartSiteTbWidget
 	for (i = 0; i < p->num_entries; ++i)
 		if (p->combos[i])
 		{
+			GList *l;
+
 			history = gb_smart_site_get_history (ss, i);
 			history_l = gslist_to_glist (history);
 			history_l = g_list_prepend (history_l, "");
 
-			gtk_combo_set_popdown_strings (GTK_COMBO (p->combos[i]), history_l);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(p->combos[i]), -1);
+			gtk_list_store_clear(GTK_LIST_STORE(
+			    gtk_combo_box_get_model(GTK_COMBO_BOX(p->combos[i]))));
+
+			for (l = history_l; l; l = l->next)
+			{
+				gtk_combo_box_append_text(GTK_COMBO_BOX(p->combos[i]),
+				                          (const gchar *)l->data);
+			}
+
 			g_slist_foreach (history, (GFunc) g_free, NULL);
 			g_slist_free (history);
 			g_list_free (history_l);
@@ -465,3 +586,42 @@ gb_smart_site_tb_widget_conf_changed_cb (GConfClient* client,
 	}
 }
 
+
+
+static void
+gb_smart_site_tb_widget_drag_data_received_cb (GtkWidget *widget,
+					       GdkDragContext *dc,
+					       gint x, gint y,
+					       GtkSelectionData *selection_data,
+					       guint info,
+					       guint t,
+					       GbSmartSiteTbWidget *w)
+{
+	GbSmartSite *ss = GB_SMART_SITE (gb_tb_widget_get_bookmark (GB_TB_WIDGET (w)));
+	gchar *url, *str;
+	gchar *args[2];
+
+	g_return_if_fail (gb_smart_site_get_num_fields (ss) == 1);
+
+	if (info == GALEON_DND_TEXT)
+	{
+		str = g_strdup (selection_data->data);
+	} 
+	else
+	{
+		str = gtk_selection_data_get_text (selection_data);
+	}
+
+	if (str == NULL || str[0] == '\0') return;
+
+	args[0] = str;
+	args[1] = NULL;
+
+	gb_smart_site_prepend_history (ss, 0, str);
+
+	url = gb_smart_site_subst_args (ss, args);
+	gb_activated_activate (w, GB_BOOKMARK (ss), url, GB_BAF_DEFAULT);
+	g_free (url);
+	g_free (str);
+}
+
