@@ -160,30 +160,33 @@ extern "C" {
 #include "gul-general.h"
 #include "gul-gui.h"
 #include "galeon-embed-utils.h"
-#include "ProgressListener.h"
 #include "ContentHandler.h"
-#include "disclosure-widget.h"
+#include "gul-file-chooser.h"
 
 #include <gtk/gtkentry.h>
-#include <gtk/gtktogglebutton.h>
 #include <gtk/gtkprogress.h>
-#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkliststore.h>
+#include <gtk/gtkwindow.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkexpander.h>
 #include <libgnome/gnome-exec.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
-#include "FilePicker.h"
 #include "MozillaPrivate.h"
+#include "ProgressListener.h"
 
 #include "nsCRT.h"
 #include "nsCOMPtr.h"
-#include "nsIFactory.h"
 #include "nsISupportsArray.h"
 #include "nsIServiceManager.h"
 #include "nsWeakReference.h"
-#include "nsXPComFactory.h"
+#include "nsIInterfaceRequestorUtils.h"
 
 #include "nsString.h"
 #include "nsIURI.h"
@@ -197,7 +200,6 @@ extern "C" {
 #include "nsIDOMWindowInternal.h"
 
 class GContentHandler;
-class GDownloadProgressListener;
 struct MimeAskActionDialog;
 struct HelperAppChooserDialog;
 
@@ -226,10 +228,7 @@ struct MimeAskActionDialog
 	GladeXML *mGXml;
 	GtkWidget *mDialogWidget;
 	GtkWidget *mParent;
-	GtkWidget *mAppMenu;
-	GtkWidget *mToggleButton;
 	GtkWidget *mListView;
-	GtkWidget *mScrolledView;
 
 	GList *mApps;
 	GtkListStore *mListStore;
@@ -247,20 +246,14 @@ struct MimeAskActionDialog
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
-GContentHandler::GContentHandler() : mUri(nsnull),
-				     mMimeType(nsnull),
-				     mDownloadCanceled(PR_FALSE),
-				     mHelperProgress(PR_FALSE)
+GContentHandler::GContentHandler() : mUri(nsnull)
 {
-	NS_INIT_ISUPPORTS();
 	/* member initializers and constructor code */
 }
 
 GContentHandler::~GContentHandler()
 {
 	/* destructor code */
-	g_free (mUri);
-	g_free (mMimeType);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -269,8 +262,16 @@ GContentHandler::~GContentHandler()
 
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
-				    nsISupports *aContext)
+				    nsISupports *aContext
+#if MOZILLA_SNAPSHOT > 9
+, PRBool aForced)
+#else
+)
+#endif
 {
+	/* aForced reflects if the content being sent is normally viewable
+	 * in mozilla or not. That fact doesn't affect us, so ignore it
+	 */
 	nsresult rv;
 
 	mLauncher = aLauncher;
@@ -283,129 +284,69 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 }
 
 /* nsILocalFile promptForSaveToFile (in nsISupports aWindowContext, in wstring aDefaultFile, in wstring aSuggestedFileExtension); */
-NS_IMETHODIMP GContentHandler::
-		PromptForSaveToFile(nsISupports *aWindowContext,
-				    const PRUnichar *aDefaultFile,
-				    const PRUnichar *aSuggestedFileExtension,
-				    nsILocalFile **_retval)
+NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
+#if MOZILLA_SNAPSHOT > 10
+						   nsIHelperAppLauncher *aLauncher,
+#endif
+						   nsISupports *aWindowContext,
+						   const PRUnichar *aDefaultFile,
+						   const PRUnichar *aSuggestedFileExtension,
+						   nsILocalFile **_retval)
 {
-	nsresult rv;
-
 	mContext = aWindowContext;
 
-	nsCOMPtr<nsIDOMWindowInternal> windowInternal = 
-					do_QueryInterface (aWindowContext);
-	
-	nsCOMPtr<nsILocalFile> saveDir;
+	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_GetInterface (aWindowContext);
+	GtkWidget *parentWindow = GTK_WIDGET (MozillaFindGtkParent (parentDOMWindow));
 
-	/* first, get the this-session-only download dir from the pref */
-	gchar *dirName = eel_gconf_get_string (CONF_STATE_LAST_DOWNLOAD_DIR);
-	if (dirName && dirName[0] == '\0')
-	{
-		g_free (dirName);
-		dirName = NULL;
-	}
-
-	/* if we don't have a dir yet, get the normal pref */
-	if (!dirName) dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
-
-	/* if we still don't have a dir, use the home directory */
-	if (!dirName || dirName[0] == '\0')
-	{
-		if (dirName) g_free (dirName);
-		dirName = g_strdup (g_get_home_dir());
-	}
-
-	/* expand the tilde, if there is one */
-	gchar *expanded_directory = gul_string_expand_home_dir (dirName);
-	g_free (dirName);
-
-	/* make sure the directory exists, and use the home directory
-	 * otherwise */
-	if (!expanded_directory ||
-	    !g_file_test (expanded_directory, G_FILE_TEST_IS_DIR))
-	{
-		if (expanded_directory) g_free (expanded_directory);
-		expanded_directory = g_strdup (g_get_home_dir());
-	}
-
-	saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	saveDir->InitWithPath (NS_ConvertUTF8toUCS2(expanded_directory));
-	g_free (expanded_directory);
-
-	nsCOMPtr <nsILocalFile> saveFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
-
-	// If we aren't asking for downloading dir, check that we aren't
-	// overwriting anything.  If we are, pop up a warning dialog and show
-	// the filepicker if the user doesn't want to overwrite the file.
-	PRInt16 okToSave = nsIFilePicker::returnCancel;
+	/* Get the directory to use for downloading into */
 	if (!eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
 	{
-		nsAutoString uDirName;
-		saveDir->GetPath (uDirName);
+		nsresult rv =  BuildDownloadPath (NS_ConvertUCS2toUTF8 (aDefaultFile).get(),
+						  parentWindow, _retval);
+		if (NS_SUCCEEDED (rv)) return rv;
+	}
 
-		const nsACString &cDirName = NS_ConvertUCS2toUTF8(uDirName);
-		const nsACString &cDefaultFile = NS_ConvertUCS2toUTF8(aDefaultFile);
-		char *filename = 
-			g_build_filename (PromiseFlatCString(cDirName).get(),
-					  PromiseFlatCString(cDefaultFile).get(),
-					  NULL);
+	GulFileChooser *dialog;
+	dialog = gul_file_chooser_new (_("Save"), parentWindow,
+					GTK_FILE_CHOOSER_ACTION_SAVE,
+					CONF_STATE_LAST_DOWNLOAD_DIR);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
+					   NS_ConvertUCS2toUTF8 (aDefaultFile).get());
 
-		// FIXME determine the parent widget if possible
-		if (gul_gui_confirm_overwrite_file (NULL, filename))
+	while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+	{
+		gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		if (!filename)
 		{
-			saveFile->InitWithPath (uDirName + NS_LITERAL_STRING("/") +
-					        nsDependentString(aDefaultFile));
-			okToSave = nsIFilePicker::returnOK;
+			continue;
 		}
 
+		if (gul_gui_confirm_overwrite_file (GTK_WIDGET (dialog),
+						    filename) == FALSE)
+		{
+			g_free (filename);
+			continue;
+		}
+
+		nsCOMPtr <nsILocalFile> destFile;
+
+		NS_NewNativeLocalFile (nsDependentCString (filename),
+				       PR_TRUE,
+				       getter_AddRefs(destFile));
 		g_free (filename);
-	}
 
-	if (okToSave == nsIFilePicker::returnCancel)
-	{
-		nsCOMPtr<nsIFilePicker> filePicker =
-					do_CreateInstance (G_FILEPICKER_CONTRACTID);
+		NS_ADDREF (*_retval = destFile);
 
-		const nsAString &title = NS_ConvertUTF8toUCS2(_("Select the destination filename"));
+		gtk_widget_destroy (GTK_WIDGET (dialog));
 
-		filePicker->Init (windowInternal,
-				   PromiseFlatString(title).get(), 
-				   nsIFilePicker::modeSave);
-		filePicker->SetDefaultString (aDefaultFile);
-		filePicker->SetDisplayDirectory (saveDir);
-
-		filePicker->Show (&okToSave);
-
-		if (okToSave == nsIFilePicker::returnOK)
-		{
-			filePicker->GetFile (getter_AddRefs(saveFile));
-		}
-	}
-
-	if (okToSave == nsIFilePicker::returnCancel)
-		return NS_ERROR_FAILURE;
-	else
-	{
-		nsCOMPtr<nsIFile> directory;
-		rv = saveFile->GetParent (getter_AddRefs(directory));
-		if(NS_SUCCEEDED(rv) && directory)
-		{
-			nsAutoString uDirectoryName;
-			rv = directory->GetPath(uDirectoryName);
-			const nsACString &cDirectoryName = NS_ConvertUCS2toUTF8(uDirectoryName);
-			if(NS_SUCCEEDED(rv))
-			{
-				eel_gconf_set_string (CONF_STATE_LAST_DOWNLOAD_DIR,
-						      PromiseFlatCString(cDirectoryName).get());
-			}
-		}
-
-		NS_IF_ADDREF (*_retval = saveFile);
 		return NS_OK;
 	}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	return NS_ERROR_FAILURE;
 }
 
+#if MOZILLA_SNAPSHOT <= 9
 /* void showProgressDialog (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 					      nsISupports *aContext)
@@ -413,6 +354,7 @@ NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 	g_print ("GContentHandler::ShowProgressDialog is depreciated!\n");
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
+#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // begin local public methods impl
@@ -446,7 +388,7 @@ NS_METHOD GContentHandler::FindHelperApp (void)
 
 NS_METHOD GContentHandler::LaunchHelperApp (void)
 {
-	if (mMimeType)
+	if (mMimeType.Length())
 	{
 		nsresult rv;
 		nsCOMPtr<nsIExternalHelperAppService> helperService =
@@ -482,12 +424,6 @@ NS_METHOD GContentHandler::LaunchHelperApp (void)
 	return NS_OK;
 }
 
-NS_METHOD GContentHandler::ShowHelperProgressDialog (void)
-{
-	mHelperProgress = PR_TRUE;
-	return ShowProgressDialog (mLauncher,mContext);
-}
-
 NS_METHOD GContentHandler::GetLauncher (nsIHelperAppLauncher * *_retval)
 {
 	NS_IF_ADDREF (*_retval = mLauncher);
@@ -537,10 +473,14 @@ NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
 	rv = mLauncher->GetMIMEInfo(getter_AddRefs(mimeInfo));
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
+	gchar *command = g_find_program_in_path (mHelperApp->command);
+	if (command == NULL) return NS_ERROR_FAILURE;
+
 	nsCOMPtr<nsILocalFile> helperFile;
-	rv = NS_NewNativeLocalFile(nsDependentCString(mHelperApp->command),
+	rv = NS_NewNativeLocalFile(nsDependentCString(command),
 				   PR_TRUE,
 				   getter_AddRefs(helperFile));
+	g_free (command);
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 	rv = mimeInfo->SetPreferredApplicationHandler(helperFile);
@@ -552,7 +492,11 @@ NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
 	if(mHelperApp->requires_terminal) //Information passing kludge!
 	{
 		rv = mimeInfo->SetApplicationDescription
+#if MOZILLA_SNAPSHOT > 16
+				(NS_LITERAL_STRING("runInTerminal"));
+#else
 				(NS_LITERAL_STRING("runInTerminal").get());
+#endif
 		if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 	}
 
@@ -571,20 +515,26 @@ NS_METHOD GContentHandler::Init (void)
 
 	nsCOMPtr<nsIMIMEInfo> MIMEInfo;
 	rv = mLauncher->GetMIMEInfo (getter_AddRefs(MIMEInfo));
-	rv = MIMEInfo->GetMIMEType (&mMimeType);
+#if MOZILLA_SNAPSHOT > 16
+	rv = MIMEInfo->GetMIMEType (mMimeType);
+#else
+	char *type = nsnull;
+	rv = MIMEInfo->GetMIMEType (&type);
+	mMimeType.Adopt (type);
+#endif
 	
+#if MOZILLA_SNAPSHOT > 11
+	rv = mLauncher->GetSource(getter_AddRefs(mUri));
+	rv = mLauncher->GetTargetFile(getter_AddRefs(mTempFile));
+#else
 	rv = mLauncher->GetDownloadInfo(getter_AddRefs(mUri),
 					&mTimeDownloadStarted,
 					getter_AddRefs(mTempFile));
+#endif
+
 	rv = mUri->GetSpec (mUrl);
 	rv = mUri->GetScheme (mScheme);
-#if 0
-	/* GetSource seems redundant and isn't in 0.9 This code is here while
-	   it remains unclear what GetSource is for. --phil */
-	nsCOMPtr<nsIURI> uri;
-	rv = mLauncher->GetSource(getter_AddRefs(uri));
-	rv = uri->GetSpec (mUrl);
-#endif	
+
 	ProcessMimeInfo ();
 
 	return NS_OK;
@@ -592,8 +542,8 @@ NS_METHOD GContentHandler::Init (void)
 
 NS_METHOD GContentHandler::ProcessMimeInfo (void)
 {
-	if (mMimeType == NULL ||
-	    !nsCRT::strcmp(mMimeType, "application/octet-stream"))
+	if (mMimeType.IsEmpty() ||
+	    mMimeType.Equals("application/octet-stream"))
 	{
 		nsresult rv;
 		nsCOMPtr<nsIURL> url = do_QueryInterface(mUri, &rv);
@@ -601,12 +551,11 @@ NS_METHOD GContentHandler::ProcessMimeInfo (void)
 		{
 			nsCAutoString uriFileName;
 			url->GetFileName(uriFileName);
-			mMimeType = g_strdup
-					(gnome_vfs_mime_type_from_name
+			mMimeType.Assign(gnome_vfs_mime_type_from_name
 						(uriFileName.get()));
 		}
 		else
-			mMimeType = g_strdup ("application/octet-stream");
+			mMimeType.Assign("application/octet-stream");
 	}
 
 	return NS_OK;
@@ -614,34 +563,10 @@ NS_METHOD GContentHandler::ProcessMimeInfo (void)
 
 NS_METHOD GContentHandler::MIMEAskAction (void)
 {
-	nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface(mContext);
+	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface(mContext);
 	GtkWidget *parentWidget = MozillaFindGtkParent(parent);
 
-	new MimeAskActionDialog(this, parentWidget, mMimeType);
-
-	return NS_OK;
-}
-
-//------------------------------------------------------------------------------
-
-NS_DEF_FACTORY (GContentHandler, GContentHandler);
-
-/**
- * NS_NewContentHandlerFactory:
- */ 
-nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsGContentHandlerFactory *result = new nsGContentHandlerFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
+	new MimeAskActionDialog(this, parentWidget, mMimeType.get());
 
 	return NS_OK;
 }
@@ -657,27 +582,38 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 					 mParent(aParentWidget)
 {
 	GtkWidget *label;
-	const char *description;
-	char ltext[255]; //philipl: Fixed length buffer == potential security problem...
-
-	mGXml = gul_glade_widget_new("galeon.glade", "mime_ask_action_dialog", 
-				     &mDialogWidget, this);
-
-	mAppMenu = glade_xml_get_widget(mGXml, "mime_ask_dialog_app_menu");
-	mToggleButton = glade_xml_get_widget(mGXml, "mime_ask_dialog_toggle_helpers");
-	mListView = glade_xml_get_widget(mGXml, "mime_ask_dialog_helper_list_view");
-	mScrolledView = glade_xml_get_widget(mGXml, "mime_ask_dialog_scrolled_view");
-
-	cddb_disclosure_set_labels (CDDB_DISCLOSURE (mToggleButton),
-			            _("Helper Applications"),
-			            _("Helper Applications"));
+	char *markup;
 
 	mApps = gnome_vfs_mime_get_all_applications(aMimeType);
 	if (mApps)
 	{
+		mGXml = gul_glade_widget_new("galeon.glade",
+					     "mime_ask_open_dialog", 
+					     &mDialogWidget, this);
+
+		mListView = glade_xml_get_widget(mGXml, "mime_ask_dialog_helper_list_view");
+
 		GnomeVFSMimeApplication *defaultApp =
 			gnome_vfs_mime_get_default_application(aMimeType);
 
+		char *title;
+		if (defaultApp)
+		{
+			title = g_strdup_printf (_("Open this file with \"%s\"?"), defaultApp->name);
+		}
+		else
+		{
+			// can this actually happen?
+			title = g_strdup (_("Open this file with another application?"));
+		}
+		markup = g_strdup_printf ("<span size=\"larger\" weight=\"bold\">%s</span>", title);
+		g_free (title);
+
+		label = glade_xml_get_widget(mGXml, "mime_ask_action_title");
+		gtk_label_set_markup (GTK_LABEL(label), markup);
+		g_free (markup);
+
+
 		mListStore = gtk_list_store_new(N_COLUMNS,
 						G_TYPE_STRING);
 		gtk_tree_view_set_model(GTK_TREE_VIEW(mListView),
@@ -727,27 +663,57 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 	}
 	else
 	{
+		mGXml = gul_glade_widget_new("galeon.glade",
+					     "mime_ask_save_dialog", 
+					     &mDialogWidget, this);
 		GtkWidget *w;
-		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_choice_label");
-		gtk_widget_hide(w);
-
-		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_open");
-		gtk_widget_set_sensitive(w, FALSE);
-
-		gtk_widget_set_sensitive(mToggleButton, FALSE);
-	}	
+		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_save");
+		gtk_widget_grab_default (w);
+	}
 
 	GtkWidget *mimeIcon = glade_xml_get_widget(mGXml,
 						   "mime_ask_action_icon");
-	gtk_image_set_from_file(GTK_IMAGE(mimeIcon),
-				gnome_vfs_mime_get_icon(aMimeType));						     
+	gul_gui_image_set_from_mime_type (mimeIcon, 
+			                  aMimeType, GTK_ICON_SIZE_DIALOG);
 
-	description = gnome_vfs_mime_get_description(aMimeType);
+
+	nsCOMPtr<nsIHelperAppLauncher> launcher;
+	(void)mContentHandler->GetLauncher(getter_AddRefs(launcher));
+
+	// suggested filename
+#if MOZILLA_SNAPSHOT <= 16
+	PRUnichar *filename = nsnull;
+	launcher->GetSuggestedFileName(&filename);
+	markup = g_strdup_printf ("<b>%s</b>", NS_ConvertUCS2toUTF8(filename).get());
+	nsMemory::Free(filename);
+#else
+        nsAutoString filename;
+        launcher->GetSuggestedFileName (filename);
+	markup = g_strdup_printf ("<b>%s</b>", NS_ConvertUCS2toUTF8(filename).get());
+#endif
+
+	label = glade_xml_get_widget(mGXml, "mime_ask_action_name");
+	gtk_label_set_markup(GTK_LABEL(label), markup);
+	g_free (markup);
+
+	// MIME type description
+	const char *description = gnome_vfs_mime_get_description(aMimeType);
 	if (!description) description = aMimeType;
 	
-	g_snprintf(ltext, 255, "<b>%s</b>", description);
 	label = glade_xml_get_widget(mGXml, "mime_ask_action_description");
-	gtk_label_set_markup(GTK_LABEL(label), ltext);
+	gtk_label_set_text(GTK_LABEL(label), description);
+
+	// source
+	nsCOMPtr<nsIURI> source;
+	launcher->GetSource(getter_AddRefs(source));
+
+	nsCAutoString spec;
+	source->Resolve(NS_LITERAL_CSTRING("."), spec);
+
+	markup = g_strdup_printf ("<i>%s</i>", PromiseFlatCString(spec).get());
+	label = glade_xml_get_widget(mGXml, "mime_ask_action_source");
+	gtk_label_set_markup(GTK_LABEL(label), markup);
+	g_free (markup);
 
 	gtk_window_set_transient_for(GTK_WINDOW(mDialogWidget), 
 				     GTK_WINDOW(aParentWidget));
@@ -771,8 +737,7 @@ MimeAskActionDialog::~MimeAskActionDialog()
 extern "C" void
 mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 {
-	gtk_widget_hide(glade_xml_get_widget(dialog->mGXml, 
-					     "mime_ask_action_dialog"));
+	gtk_widget_hide(dialog->mDialogWidget);
 
 	nsresult rv;
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
@@ -852,19 +817,21 @@ mime_ask_dialog_cancel_clicked_cb(GtkButton *button,
 }
 
 extern "C" void
-mime_ask_dialog_toggle_helpers_toggled_cb(GtkToggleButton *button,
+mime_ask_dialog_toggle_helpers_toggled_cb(GtkExpander *expander,
+					  GParamSpec *pspec,
 					  MimeAskActionDialog *dialog)
 {
-	if (gtk_toggle_button_get_active(button))
-	{
-		gtk_widget_show(dialog->mScrolledView);
-		gtk_window_set_resizable(GTK_WINDOW(dialog->mDialogWidget),
-					 TRUE);
-	}
-	else
-	{
-		gtk_window_set_resizable(GTK_WINDOW(dialog->mDialogWidget),
-					 FALSE);
-		gtk_widget_hide(dialog->mScrolledView);
-	}
+	gboolean val = gtk_expander_get_expanded (expander);
+	gtk_window_set_resizable (GTK_WINDOW(dialog->mDialogWidget), val);
 }
+
+extern "C" void
+mime_ask_dialog_close_cb(GtkWidget *dialogWidget, MimeAskActionDialog *dialog)
+{
+	nsresult rv;
+	nsCOMPtr<nsIHelperAppLauncher> launcher;
+	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+
+	launcher->Cancel();
+}
+
