@@ -20,23 +20,25 @@
 #include "config.h"
 #endif
 
-#define DEBUG_MSG(x) g_print x
-//#define DEBUG_MSG(x)
-
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
 
 #include "galeon-tab.h"
 #include "galeon-macros.h"
 #include "galeon-shell.h"
-#include "galeon-embed-popup-bw.h"
+#include "galeon-popup.h"
+#include "galeon-embed-utils.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
 #include "galeon-embed-prefs.h"
 #include "galeon-embed-autoscroller.h"
 #include "galeon-embed-manual-scroller.h"
+#include "gul-download.h"
+#include "gul-notebook.h"
 #include "gul-gestures.h"
+#include "galeon-embed-favicon.h"
+#include "galeon-debug.h"
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkbutton.h>
@@ -47,25 +49,29 @@
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkstyle.h>
 #include <gtk/gtkselection.h>
+#include <gtk/gtkclipboard.h>
 #include <string.h>
 
 struct GaleonTabPrivate
 {
-	GaleonEmbed *embed;
+	GList *notifiers;
 	GaleonWindow *window;
 	gboolean is_active;
+	GtkWidget *tab_icon;
 	TabLoadStatus load_status;
 	char *link_message;
 	char *js_message;
 	char status_message[255];
 	char *title;
 	char *location;
+	char *blocked_popup_uri;
 	int load_percent;
 	gboolean visibility;
 	int cur_requests;
 	int total_requests;
 	int width;
 	int height;
+	int zoom;
 };
 
 static GHashTable *gestures_actions = NULL;
@@ -77,12 +83,6 @@ galeon_tab_init (GaleonTab *tab);
 static void
 galeon_tab_finalize (GObject *object);
 
-enum
-{
-	PROP_0,
-	PROP_GALEON_SHELL
-};
-
 static void
 galeon_tab_link_message_cb (GaleonEmbed *embed, 
 			    const char *message,
@@ -103,17 +103,16 @@ static void
 galeon_tab_net_state_cb (GaleonEmbed *embed, const char *uri,
 			 EmbedState state, GaleonTab *tab);
 static void
+galeon_tab_net_stop_cb (GaleonEmbed *embed, GaleonTab *tab);
+static void
+galeon_tab_net_start_cb (GaleonEmbed *embed, GaleonTab *tab);
+static void
 galeon_tab_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
 			  EmbedChromeMask chromemask, GaleonTab *tab);
 static void
 galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 			  GaleonTab *tab);
 static void
-galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab);
-static gint
-galeon_tab_open_uri_cb (GaleonEmbed *embed, const char *uri,
-			GaleonTab *tab);
-static void
 galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 		       GaleonTab *tab);
 static gint
@@ -125,11 +124,28 @@ galeon_tab_dom_mouse_down_cb (GaleonEmbed *embed,
 			      GaleonEmbedEvent *event,
 			      GaleonTab *tab);
 static void
+galeon_tab_contextmenu_cb (GaleonEmbed *embed,
+		           GaleonEmbedEvent *event,
+			   GaleonTab *tab);
+static void
 galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 			       GaleonTab *tab);
 static void
 galeon_tab_zoom_changed_cb (GaleonEmbed *embed, gint zoom, 
 			    GaleonTab *tab);
+static void
+galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab);
+
+static void
+galeon_tab_popup_blocked_cb(GaleonEmbed *embed, const char *uri,
+			    GaleonTab *tab);
+static void
+galeon_tab_size_allocate(GtkWidget *widget, GtkAllocation *allocation);
+
+static void galeon_tab_destroy (GtkObject *object);
+
+static void
+galeon_tab_parent_set (GtkWidget *widget, GtkWidget *previous_parent);
 
 static void
 galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTab *tab);
@@ -158,7 +174,7 @@ galeon_tab_get_type (void)
                         (GInstanceInitFunc) galeon_tab_init
                 };
 
-                galeon_tab_type = g_type_register_static (G_TYPE_OBJECT,
+                galeon_tab_type = g_type_register_static (GTK_TYPE_BIN,
 							  "GaleonTab",
 							  &our_info, 0);
         }
@@ -169,36 +185,88 @@ galeon_tab_get_type (void)
 static void
 galeon_tab_class_init (GaleonTabClass *klass)
 {
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GObjectClass *object_class   = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
+	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
+	parent_class = g_type_class_peek_parent (klass);
 
-        object_class->finalize = galeon_tab_finalize;
+	widget_class->size_allocate = galeon_tab_size_allocate;
+	widget_class->parent_set    = galeon_tab_parent_set;
+	gtk_object_class->destroy   = galeon_tab_destroy;
+	object_class->finalize = galeon_tab_finalize;
+}
+
+
+static void
+galeon_tab_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
+{
+	GtkWidget *child;
+
+	g_return_if_fail(IS_GALEON_TAB (widget));
+	
+	widget->allocation = *allocation;
+	child = GTK_BIN (widget)->child;
+
+	if (child && GTK_WIDGET_VISIBLE (child))
+	{
+		gtk_widget_size_allocate (child, allocation);
+	}
 }
 
 static void
-galeon_tab_parent_set_cb (GtkWidget *widget, GtkWidget *previous_parent, 
-			  GaleonTab *tab)
+galeon_tab_destroy (GtkObject *object)
 {
-	GtkWidget *toplevel;
+	GaleonEmbed *embed;
+	
+	embed = galeon_tab_get_embed (GALEON_TAB (object));
+	if (embed)
+	{
+		/* The GaleonTab (and thus the embed) are about to be
+		 * destroyed. We therefore don't care about any
+		 * signals that the embed might emit. So we block them
+		 * all (bug #138529 - see tests/popup.html testcase)
+		 *
+		 * The one exception is the "new_window" signal,
+		 * which we do want to still listen to, otherwise
+		 * if a tab tries to launch a popup window when
+		 * it closes, we will crash. So block all, then
+		 * unblock just that signal
+		 */
+		LOG ("Blocking all signals (except new_window)");
+		g_signal_handlers_block_matched (embed, G_SIGNAL_MATCH_DATA, 
+						 0, 0, NULL, NULL, object);
 
+		/* But don't block the new_window callback */
+		g_signal_handlers_unblock_matched (embed, G_SIGNAL_MATCH_FUNC,
+						   0, 0, NULL, 
+						   galeon_tab_new_window_cb, object);
+
+	}
+	
+	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+}
+
+static void
+galeon_tab_parent_set (GtkWidget *widget, GtkWidget *previous_parent)
+{
 	/* FIXME maybe we dont need to set the tab parent explicitly
 	 * anymore with this callback taking care of it */
 	
-	if (widget->parent == NULL) return;
+	if (widget->parent)
+	{
+		GtkWidget *toplevel;
+		toplevel = gtk_widget_get_toplevel (widget);
 
-	toplevel = gtk_widget_get_toplevel (widget);
+		GALEON_TAB (widget)->priv->window = GALEON_WINDOW (toplevel);
+	}
+	else
+	{
+		GALEON_TAB (widget)->priv->is_active = FALSE;
+	}
 
-	tab->priv->window = GALEON_WINDOW (toplevel);
-}
-
-static void
-galeon_tab_embed_destroy_cb (GtkWidget *widget, GaleonTab *tab)
-{
-#ifdef DEBUG_MARCO
-	g_print ("GtkMozEmbed destroy signal on GaleonTab\n");
-#endif
-	g_object_unref (tab);
+	if (GTK_WIDGET_CLASS (parent_class)->parent_set)
+		GTK_WIDGET_CLASS (parent_class)->parent_set (widget, previous_parent);
 }
 
 static void
@@ -230,17 +298,78 @@ out:
 	g_free (url2);
 }
 
+
+static void
+tab_icon_set_visibility (GaleonFavicon *favicon)
+{
+	int mode = eel_gconf_get_integer (CONF_TABS_FAVICON);
+
+	switch (mode)
+	{
+	case 0:
+		gtk_widget_hide (GTK_WIDGET(favicon));
+		break;
+
+	case 1:
+		gtk_widget_show (GTK_WIDGET(favicon));
+		break;
+
+	default:
+		if (galeon_favicon_get_is_default (favicon))
+		{
+			gtk_widget_hide (GTK_WIDGET(favicon));
+		}
+		else
+		{
+			gtk_widget_show (GTK_WIDGET(favicon));
+		}
+		break;
+	}
+}
+
+
+static void
+tab_icon_changed_cb (GaleonFavicon *favicon, gpointer dummy)
+{
+	tab_icon_set_visibility (favicon);
+}
+
+
+static void
+tab_favicon_mode_gconf_changed_cb (GConfClient *client, guint cnxn_id,
+				   GConfEntry *entry, GaleonTab *tab)
+{
+	tab_icon_set_visibility (GALEON_FAVICON (tab->priv->tab_icon));
+}
+
+
 static void
 galeon_tab_init (GaleonTab *tab)
 {
-	GObject *embed, *embed_widget;
+	GObject *embed;
 	GaleonEmbedShell *shell;
 	
+	LOG ("Creating tab %p", tab);
+
         tab->priv = g_new0 (GaleonTabPrivate, 1);
 
 	shell = galeon_shell_get_embed_shell (galeon_shell);
 	
-	tab->priv->embed = galeon_embed_new (G_OBJECT(shell));
+	embed = G_OBJECT (galeon_embed_new (G_OBJECT(shell)));
+
+	tab->priv->tab_icon = galeon_embed_favicon_new (GALEON_EMBED (embed));
+	g_object_ref (G_OBJECT (tab->priv->tab_icon));
+	gtk_object_sink (GTK_OBJECT (tab->priv->tab_icon));
+
+	tab_icon_set_visibility (GALEON_FAVICON (tab->priv->tab_icon));
+
+	galeon_notification_add(CONF_TABS_FAVICON,
+				(GConfClientNotifyFunc)tab_favicon_mode_gconf_changed_cb,
+				tab, &tab->priv->notifiers);
+
+	g_signal_connect_object (tab->priv->tab_icon, "changed",
+				 G_CALLBACK (tab_icon_changed_cb), 
+				 tab, 0);
 
 	tab->priv->window = NULL;
 	tab->priv->is_active = FALSE;
@@ -255,68 +384,68 @@ galeon_tab_init (GaleonTab *tab)
 	tab->priv->cur_requests = 0;
 	tab->priv->width = -1;
 	tab->priv->height = -1;
+	tab->priv->zoom   = 100;
 
-	
-	embed = G_OBJECT (tab->priv->embed);
-	embed_widget = G_OBJECT (tab->priv->embed);
+	gtk_container_add (GTK_CONTAINER (tab), GTK_WIDGET (embed));
+	gtk_widget_show (GTK_WIDGET(embed));
 
-	/* set a pointer in the embed's widget back to the tab */
-	g_object_set_data (embed_widget, "GaleonTab", tab);
-	
-	g_signal_connect (embed_widget, "parent_set",
-			  G_CALLBACK (galeon_tab_parent_set_cb), 
-			  tab);
-	g_signal_connect (embed_widget, "destroy",
-			  GTK_SIGNAL_FUNC (galeon_tab_embed_destroy_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_link_message",
-			  GTK_SIGNAL_FUNC (galeon_tab_link_message_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_js_status",
-			  GTK_SIGNAL_FUNC (galeon_tab_js_status_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_location",
-			  GTK_SIGNAL_FUNC (galeon_tab_location_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_title",
-			  GTK_SIGNAL_FUNC (galeon_tab_title_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_zoom_change",
-			  GTK_SIGNAL_FUNC (galeon_tab_zoom_changed_cb),
-			  tab);
-	g_signal_connect (embed, "ge_progress",
-			  GTK_SIGNAL_FUNC (galeon_tab_progress_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_net_state",
-			  GTK_SIGNAL_FUNC (galeon_tab_net_state_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_new_window",
-			  GTK_SIGNAL_FUNC (galeon_tab_new_window_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_visibility",
-			  GTK_SIGNAL_FUNC (galeon_tab_visibility_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_destroy_brsr",
-			  GTK_SIGNAL_FUNC (galeon_tab_destroy_brsr_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_open_uri",
-			  GTK_SIGNAL_FUNC (galeon_tab_open_uri_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_size_to",
-			  GTK_SIGNAL_FUNC (galeon_tab_size_to_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_dom_mouse_click",
-			  GTK_SIGNAL_FUNC (galeon_tab_dom_mouse_click_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_dom_mouse_down",
-			  GTK_SIGNAL_FUNC (galeon_tab_dom_mouse_down_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_security_change",
-			  GTK_SIGNAL_FUNC (galeon_tab_security_change_cb), 
-			  tab);
-	g_signal_connect (embed, "ge_favicon",
-			  G_CALLBACK (galeon_tab_favicon_cb),
-			  tab);
+	g_signal_connect_object (embed, "ge_link_message",
+				 GTK_SIGNAL_FUNC (galeon_tab_link_message_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_js_status",
+				 GTK_SIGNAL_FUNC (galeon_tab_js_status_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_location",
+				 GTK_SIGNAL_FUNC (galeon_tab_location_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_title",
+				 GTK_SIGNAL_FUNC (galeon_tab_title_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_zoom_change",
+				 GTK_SIGNAL_FUNC (galeon_tab_zoom_changed_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_progress",
+				 GTK_SIGNAL_FUNC (galeon_tab_progress_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_net_state",
+				 GTK_SIGNAL_FUNC (galeon_tab_net_state_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_net_stop",
+				 GTK_SIGNAL_FUNC (galeon_tab_net_stop_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_net_start",
+				 GTK_SIGNAL_FUNC (galeon_tab_net_start_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_new_window",
+				 GTK_SIGNAL_FUNC (galeon_tab_new_window_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_visibility",
+				 GTK_SIGNAL_FUNC (galeon_tab_visibility_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_destroy_brsr",
+				 GTK_SIGNAL_FUNC (galeon_tab_destroy_brsr_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_size_to",
+				 GTK_SIGNAL_FUNC (galeon_tab_size_to_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_dom_mouse_click",
+				 GTK_SIGNAL_FUNC (galeon_tab_dom_mouse_click_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_dom_mouse_down",
+				 GTK_SIGNAL_FUNC (galeon_tab_dom_mouse_down_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_contextmenu",
+				 GTK_SIGNAL_FUNC (galeon_tab_contextmenu_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_security_change",
+				 GTK_SIGNAL_FUNC (galeon_tab_security_change_cb), 
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_favicon",
+				 G_CALLBACK (galeon_tab_favicon_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_popupblocked",
+				 G_CALLBACK(galeon_tab_popup_blocked_cb),
+				 tab, 0);
 }
 
 /* Destructor */
@@ -324,28 +453,47 @@ galeon_tab_init (GaleonTab *tab)
 static void
 galeon_tab_finalize (GObject *object)
 {
-        GaleonTab *tab;
-
-	g_return_if_fail (IS_GALEON_TAB (object));
-
-	tab = GALEON_TAB (object);
+        GaleonTab *tab = GALEON_TAB (object);
 
         g_return_if_fail (tab->priv != NULL);
 
-	g_idle_remove_by_data (tab->priv->embed);
-	
+	galeon_notification_remove (&tab->priv->notifiers);
+
+	g_idle_remove_by_data (tab);
+
+	g_object_unref (G_OBJECT(tab->priv->tab_icon));
+
 	g_free (tab->priv->link_message);
 	g_free (tab->priv->js_message);
+	g_free (tab->priv->blocked_popup_uri);
 
+        g_free (tab->priv->title);
+        g_free (tab->priv->location);
         g_free (tab->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 
-#ifdef DEBUG_MARCO
-	g_print ("GaleonTab finalized %p\n", tab);
-#endif
+	LOG ("GaleonTab finalized %p", tab);
 }
 
+
+static gboolean
+address_has_web_scheme (const char *address)
+{
+	gboolean has_web_scheme;
+
+	if (address == NULL) return FALSE;
+
+	has_web_scheme = (g_str_has_prefix (address, "http:") ||
+			  g_str_has_prefix (address, "https:") ||
+			  g_str_has_prefix (address, "ftp:") ||
+			  g_str_has_prefix (address, "file:") ||
+			  g_str_has_prefix (address, "gopher:"));
+
+	return has_web_scheme;
+}
+
+
 /* Public functions */
 
 GaleonTab *
@@ -354,7 +502,7 @@ galeon_tab_new ()
 	GaleonTab *tab;
 
 	tab = GALEON_TAB (g_object_new (GALEON_TAB_TYPE, NULL));
-	g_return_val_if_fail (tab->priv != NULL, NULL);
+
 	return tab;
 }
 
@@ -383,9 +531,13 @@ galeon_tab_set_load_status (GaleonTab *tab,
 GaleonEmbed *
 galeon_tab_get_embed (GaleonTab *tab)
 {
-	g_return_val_if_fail (IS_GALEON_TAB (G_OBJECT (tab)), NULL);
+	GtkWidget* child;
 
-	return tab->priv->embed;
+	g_return_val_if_fail (IS_GALEON_TAB (tab), NULL);
+
+	child = gtk_bin_get_child (GTK_BIN (tab));
+
+	return child ? GALEON_EMBED (child) : NULL;
 }
 
 void
@@ -450,15 +602,27 @@ galeon_tab_get_size (GaleonTab *tab, int *width, int *height)
 	*height = tab->priv->height;
 }
 
+GtkWidget *
+galeon_tab_get_icon (GaleonTab *tab)
+{
+	return tab->priv->tab_icon;
+}
+
 static void 
 galeon_tab_set_visibility (GaleonTab *tab,
                            gboolean visible)
 {	
-	g_return_if_fail (tab->priv->window != NULL);
+	GaleonWindow *window;
+
+	g_return_if_fail (IS_GALEON_TAB (tab));
 
-	/* FIXME show/hide the tab widget */
-	
 	tab->priv->visibility = visible;
+
+	window = galeon_tab_get_window (tab);
+	g_return_if_fail (window != NULL);
+	
+	galeon_window_update_control (window, WindowVisibilityControl);
+
 }
 
 /* Private callbacks for embed signals */
@@ -507,10 +671,32 @@ galeon_tab_location_cb (GaleonEmbed *embed, GaleonTab *tab)
 	galeon_embed_get_location(embed, TRUE, FALSE, 
 				  &tab->priv->location);
 
+	/* restore zoom level, but only for pages that are from the
+	 * Internet, i.e. not javascript, about or data pages */
+	if (address_has_web_scheme (tab->priv->location))
+	{
+		GlobalHistory *history;
+		GaleonEmbedShell *ges;
+		gint saved_zoom, current_zoom;
+		gresult ret;
+
+		ges = galeon_shell_get_embed_shell (galeon_shell);
+		history = galeon_embed_shell_get_global_history (ges);
+
+		saved_zoom = global_history_get_host_zoom (history, tab->priv->location);
+		ret = galeon_embed_zoom_get (embed, &current_zoom);
+		if (ret == G_OK && saved_zoom > 0 && current_zoom != saved_zoom)
+		{
+			g_signal_handlers_block_by_func (embed, galeon_tab_zoom_changed_cb, tab);
+			tab->priv->zoom = saved_zoom;
+			galeon_embed_zoom_set (embed, saved_zoom, FALSE);
+			g_signal_handlers_unblock_by_func (embed, galeon_tab_zoom_changed_cb, tab);
+		}
+	}
+
 	if (tab->priv->is_active)
 	{
 		galeon_window_update_control(tab->priv->window, LocationControl);
-		galeon_window_update_control(tab->priv->window, NavControl);
 		galeon_window_update_control(tab->priv->window, PermissionControl);
 	}
 
@@ -521,11 +707,17 @@ static void
 galeon_tab_zoom_changed_cb (GaleonEmbed *embed, gint zoom, GaleonTab *tab)
 {
 	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
-	
-	if (tab->priv->is_active)
+	tab->priv->zoom = zoom;
+
+	if (address_has_web_scheme (tab->priv->location))
 	{
-		galeon_window_update_control (tab->priv->window, ZoomControl);
-	}
+		GlobalHistory *history;
+		GaleonEmbedShell *ges;	
+
+		ges = galeon_shell_get_embed_shell (galeon_shell);
+		history = galeon_embed_shell_get_global_history (ges);
+		global_history_set_host_zoom (history, tab->priv->location, zoom);
+	} 
 }
 
 static void
@@ -539,7 +731,14 @@ galeon_tab_title_cb (GaleonEmbed *embed, GaleonTab *tab)
 	if (*(tab->priv->title) == '\0')
 	{
 		g_free (tab->priv->title);
-		tab->priv->title = g_strdup (_("Untitled"));
+		if (!strcmp (tab->priv->location, "about:blank"))
+		{
+			tab->priv->title = g_strdup(_("Untitled"));
+		}
+		else
+		{
+			tab->priv->title = g_strdup (tab->priv->location);
+		}
 	}
 	
 	galeon_window_site_visited (tab->priv->window, tab->priv->location, tab->priv->title);
@@ -674,7 +873,7 @@ build_net_state_message (char *message,
                 }
                 else if (flags & EMBED_STATE_STOP)
                 {
-                      	msg = _("Done.");
+                      	msg = " ";
                 }
         }
 
@@ -707,69 +906,66 @@ build_progress_from_requests (GaleonTab *tab, EmbedState state)
 		if (load_percent > tab->priv->load_percent)
 		{
 			tab->priv->load_percent = load_percent;
+
+			galeon_window_update_control (tab->priv->window,
+						      StatusbarProgressControl);
 		}
 	}
 }
 
 static void
-ensure_location (GaleonTab *tab, const char *uri)
+galeon_tab_net_start_cb (GaleonEmbed *embed, GaleonTab *tab)
 {
-	if (tab->priv->location == NULL)
+	tab->priv->total_requests = 0;
+	tab->priv->cur_requests   = 0;
+	tab->priv->load_percent   = 0;
+
+	galeon_tab_set_load_status (tab, TAB_LOAD_STARTED);
+	galeon_tab_update_color (tab);
+	
+	if (tab->priv->blocked_popup_uri)
 	{
-		tab->priv->location = g_strdup (uri);
-		galeon_window_update_control (tab->priv->window,
-					      LocationControl);
+		g_free(tab->priv->blocked_popup_uri);
+		tab->priv->blocked_popup_uri = NULL;
+	}
+
+	galeon_window_update_control (tab->priv->window, SpinnerControl);
+
+	if (tab->priv->is_active)
+	{
+		galeon_window_update_control (tab->priv->window, StatusbarPopupBlockControl);
 	}
 }
 
+
 static void
 galeon_tab_net_state_cb (GaleonEmbed *embed, const char *uri,
 			 EmbedState state, GaleonTab *tab)
 {	
 	build_net_state_message (tab->priv->status_message, uri, state);
-
+	build_progress_from_requests (tab, state);
+	
 	if (tab->priv->is_active)
 	{
 		galeon_window_update_control (tab->priv->window,
 					      StatusbarMessageControl);
-	}
-	
-	if (state & EMBED_STATE_IS_NETWORK)
-	{
-		if (state & EMBED_STATE_START)
-		{
-			tab->priv->total_requests = 0;
-			tab->priv->cur_requests = 0;
-			tab->priv->load_percent = 0;
-		
-			ensure_location (tab, uri);
-			
-			galeon_tab_set_load_status (tab, TAB_LOAD_STARTED);
 
-			galeon_window_update_control (tab->priv->window,
-						      NavControl);
-			galeon_window_update_control (tab->priv->window,
-						      SpinnerControl);
-			galeon_tab_update_color (tab);
-		}
-		else if (state & EMBED_STATE_STOP)
-		{			
-			tab->priv->load_percent = 0;
+	}
+}
+
+static void
+galeon_tab_net_stop_cb (GaleonEmbed *embed, GaleonTab *tab)
+{
+	tab->priv->load_percent = 0;
 			
-			galeon_tab_set_load_status (tab, TAB_LOAD_COMPLETED);
+	galeon_tab_set_load_status (tab, TAB_LOAD_COMPLETED);
 			
-			galeon_window_update_control (tab->priv->window,
-						      NavControl);
-			galeon_window_update_control (tab->priv->window,
-						      SpinnerControl);
-			galeon_tab_update_color (tab);
-		}		
-	}	
-	
-	build_progress_from_requests (tab, state);
-	
-	galeon_window_update_control (tab->priv->window,
-				      StatusbarProgressControl);
+	galeon_window_update_control (tab->priv->window, SpinnerControl);
+	if (tab->priv->is_active)
+	{
+		galeon_window_update_control (tab->priv->window, StatusbarProgressControl);
+	}
+	galeon_tab_update_color (tab);
 }
 
 static void
@@ -795,59 +991,78 @@ galeon_tab_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
 	}
 	
 	new_tab = galeon_tab_new ();
-        galeon_window_add_tab (window, new_tab, FALSE);
+	gtk_widget_show (GTK_WIDGET (new_tab));
+        galeon_window_add_tab (window, new_tab,
+			       GALEON_WINDOW_ADD_TAB_GROUPED, FALSE);
 	
 	*new_embed = galeon_tab_get_embed (new_tab);
 }
 
-static gboolean
-let_me_resize_hack (gpointer data)
-{
-	gtk_widget_set_size_request (GTK_WIDGET(data), 
-				     -1, -1);
-	return FALSE;
-}
-
 static void
 galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 			  GaleonTab *tab)
 {
-	GaleonWindow *window;
-	
 	if (visibility)
 	{
-		gtk_widget_show (GTK_WIDGET(embed));
+		gtk_widget_show (GTK_WIDGET (tab));
 	}
 	else
 	{
-		gtk_widget_hide (GTK_WIDGET(embed));
+		gtk_widget_hide (GTK_WIDGET (tab));
 	}
         
 	galeon_tab_set_visibility (tab, visibility);
-
-	window = galeon_tab_get_window (tab);
-	g_return_if_fail (window != NULL);
-	
-	galeon_window_update_control (window, WindowVisibilityControl);
-#if MOZILLA_SNAPSHOT <= 6
-	galeon_window_update_control(window, PermissionControl);
-#endif
 }
 
+
+static gboolean
+galeon_tab_destroy_brsr_idle_cb (GaleonTab *tab)
+{
+	GulNotebook *nb;
+	g_return_val_if_fail (IS_GALEON_TAB (tab), FALSE);
+	
+	LOG ("galeon_tab_destroy_brsr_idle_cb");
+
+	/* Don't use galeon_window_remove_tab here, we don't
+	 * want to run the "unsubmitted" changes code */
+	nb = galeon_window_get_notebook (tab->priv->window);
+	gul_notebook_remove_page (GUL_NOTEBOOK (nb), GTK_WIDGET (tab));
+	return FALSE;
+}
+
+
 static void
 galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab)
 {
-	GaleonWindow *window;
+	/* This is a dirty to hack to work around mozillas problems
+	 * with closing windows via javascript that dont have focus.
+	 * 
+	 * The problem is that if you have a tab loaded in the
+	 * background in the notebook, then under some circumstances
+	 * mozilla won't emit the ge_destroy_brsr signal until it gets
+	 * focus by someone clicking on the tab label.  At that point,
+	 * the signal is emitted, we close the tab, remove the label,
+	 * and then the notebook crashes because widget for the label
+	 * has been destroyed.
+	 *
+	 * See bug #116256
+	 *
+	 * So lets work around the problem by adding an g_idle function
+	 * that destroys the tab when we know nothing is referencing it
+	 * in the stack.
+	 */
+	LOG ("galeon_tab_destroy_brsr_cb");
 
-	window = galeon_tab_get_window (tab);
-
-	galeon_window_remove_tab (window, tab);
+	g_idle_add_full (G_PRIORITY_HIGH_IDLE, 
+			 (GSourceFunc)galeon_tab_destroy_brsr_idle_cb, tab, 0);
 }
 
-static gint
-galeon_tab_open_uri_cb (GaleonEmbed *embed, const char *uri,
-			GaleonTab *tab)
+static gboolean
+let_me_resize_hack (gpointer data)
 {
+	g_return_val_if_fail (IS_GALEON_TAB (data), FALSE);
+
+	gtk_widget_set_size_request (GTK_WIDGET(data), -1, -1);
 	return FALSE;
 }
 
@@ -857,7 +1072,6 @@ galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 {
 	GList *tabs;
 	GaleonWindow *window;
-	GtkWidget *widget;
 	EmbedChromeMask chromemask;
 
 	tab->priv->width = width;
@@ -865,7 +1079,6 @@ galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 	
 	window = galeon_tab_get_window (tab);
 	tabs = (GList *) galeon_window_get_tabs (window);
-	widget = GTK_WIDGET (embed);
 	chromemask = galeon_window_get_chrome (window);
 	
 	/* Do not resize window with multiple tabs.
@@ -875,7 +1088,7 @@ galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 	if (g_list_length (tabs) == 1 && !tab->priv->visibility)
 	{
 		gtk_widget_set_size_request 
-			(widget, width, height);
+			(GTK_WIDGET (tab), width, height);
 
 		/* HACK reset widget requisition after the container
 		 * has been resized. It appears to be the only way
@@ -886,7 +1099,7 @@ galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 		 */
 		if (!(chromemask & EMBED_CHROME_OPENASCHROME))
 		{
-			g_idle_add (let_me_resize_hack, embed);
+			g_idle_add (let_me_resize_hack, tab);
 		}
 	}
 }
@@ -894,16 +1107,16 @@ galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 static void
 galeon_tab_show_embed_popup (GaleonTab *tab, GaleonEmbedEvent *event)
 {
-	GaleonEmbedPopup *popup;
+	GaleonPopup *popup;
 	GaleonWindow *window;
 	GaleonEmbed *embed;
 
 	window = galeon_tab_get_window (tab);
 	embed = galeon_tab_get_embed (tab);
 
-	popup = GALEON_EMBED_POPUP (galeon_window_get_popup_factory (window));
-	galeon_embed_popup_set_event (popup, event);
-	galeon_embed_popup_show (popup, embed);
+	popup = GALEON_POPUP (galeon_window_get_popup_factory (window));
+	galeon_popup_set_event (popup, event);
+	galeon_popup_show (popup, embed);
 }
 
 static gint
@@ -939,7 +1152,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 					       g_object_ref (event), 
 					       g_object_unref);
 			
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
 			
 			g_signal_connect (g, "gesture-performed", 
@@ -974,7 +1187,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		{
 			GulGestures *g = gul_gestures_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
 			
 			g_signal_connect (g, "gesture-performed", 
@@ -987,7 +1200,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		{
 			GaleonEmbedAutoscroller *as = galeon_embed_autoscroller_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			galeon_embed_autoscroller_set_embed (as, embed);
 			galeon_embed_autoscroller_start_scroll (as, GTK_WIDGET (window), x, y);
 			g_object_unref (as);
@@ -997,7 +1210,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		{
 			GaleonEmbedManualScroller *ms = galeon_embed_manual_scroller_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			galeon_embed_manual_scroller_set_embed (ms, embed);
 			galeon_embed_manual_scroller_start_scroll (ms, GTK_WIDGET (window), x, y);
 			g_object_unref (ms);
@@ -1012,13 +1225,51 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 	return FALSE;
 }
 
+/**
+ *  Callback function from the paste of text.
+ */
+static void
+galeon_tab_clipboard_text_received_cb (GtkClipboard *clipboard, const gchar *text,
+				       gpointer data)
+{
+	gpointer *weak_ptr = (gpointer*)data;
+	GaleonTab *tab = NULL;
+	gint action;
+
+	if (*weak_ptr != NULL)
+	{
+		tab = GALEON_TAB (*weak_ptr);
+		g_object_remove_weak_pointer (G_OBJECT (tab), weak_ptr);
+	}
+	g_free (data);
+
+	if (tab == NULL || text == NULL || text[0] == '\0')
+	{
+		return;
+	}
+
+	action = eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION);
+	
+	if (action == 6) /* Paste into current tab */
+	{
+		galeon_embed_load_url (galeon_tab_get_embed (tab), text);
+	}
+	else /* Paste into new tab (action == 1 or default) */
+	{
+		GaleonWindow * window = galeon_tab_get_window (tab);
+
+		galeon_shell_new_tab(galeon_shell, window, tab,
+ 				     text, 0);
+	}
+}
+
 static gint
 galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 			      GaleonEmbedEvent *event,
 			      GaleonTab *tab)
 {
 	GaleonWindow *window;
-	int button;
+	guint button, modifier;
 	EmbedEventContext context;
         
 	g_assert(IS_GALEON_EMBED_EVENT(event));
@@ -1027,18 +1278,51 @@ galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 	g_return_val_if_fail(window != NULL, FALSE);
 
 	galeon_embed_event_get_mouse_button(event, &button);
-	galeon_embed_event_get_context(event, &context); 
+	galeon_embed_event_get_context(event, &context);
+	galeon_embed_event_get_modifier(event, &modifier);
 
-	if (button == 1 && (context & EMBED_CONTEXT_LINK))
+	if (button == 0 && (context & EMBED_CONTEXT_LINK) &&
+	    !(context & EMBED_CONTEXT_INPUT) &&
+	    (modifier & GDK_SHIFT_MASK))
 	{
-		GaleonNewTabFlags flags;
+		galeon_embed_utils_download_event_property (embed, event, 
+							    FALSE, NULL, "link");
+	}
+	else if (button == 0 && (context & EMBED_CONTEXT_IMAGE) &&
+		 !(context & EMBED_CONTEXT_INPUT) &&
+		 (modifier & GDK_SHIFT_MASK))
+	{
+		galeon_embed_utils_save_event_property(embed, event, 
+						       TRUE,  /* always_ask_dir */
+						       FALSE, /* show_progress */
+						       _("Save Image As..."),
+						       "image");
+	}
+	else if ((button == 0 && (context & EMBED_CONTEXT_LINK) &&
+		  !(context & EMBED_CONTEXT_INPUT) &&
+		  (modifier & GDK_CONTROL_MASK)) ||
+		 (button == 1 && (context & EMBED_CONTEXT_LINK)))
+	{
+		GaleonNewTabFlags flags = 0;
 		const GValue *value;
+		const gchar *link;
 
-		flags = galeon_shell_modifier_flags(galeon_embed_event_get_modifier (event));
+		if (button == 1)
+		{
+			flags = galeon_shell_modifier_flags (modifier);
+		}
 		galeon_embed_event_get_property(event, "link", &value);
-		galeon_shell_new_tab(galeon_shell, window, tab, 
-				     g_value_get_string (value), 
-				     flags);
+		link = g_value_get_string (value);
+
+		if (galeon_embed_event_has_property(event, "link-can-open-in-new-tab"))
+		{
+			galeon_shell_new_tab(galeon_shell, window, tab, 
+					     link, flags);
+		}
+		else
+		{
+			galeon_embed_load_url (galeon_tab_get_embed (tab), link);
+		}
 	}
 	else if (button == 1 &&
 		 (eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION) == 1 ||
@@ -1047,14 +1331,29 @@ galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 		   context & EMBED_CONTEXT_EMAIL_LINK ||
 		   context & EMBED_CONTEXT_INPUT))
 	{
-		gtk_selection_convert(GTK_WIDGET(window), 
-				      GDK_SELECTION_PRIMARY,
-				      GDK_SELECTION_TYPE_STRING,
-				      GDK_CURRENT_TIME);
+		/* Handle UTF8 characters as well as normal variants, see bug 133633 */
+
+		/* Ensure that if the tab is destroyed while we are requesting text,
+		 * we don't dereference  a bogus tab pointer */
+		gpointer * weak_ptr = g_new (gpointer, 1);
+		*weak_ptr = tab;
+		g_object_add_weak_pointer (G_OBJECT (tab), weak_ptr);
+
+		gtk_clipboard_request_text (gtk_widget_get_clipboard (GTK_WIDGET (embed),
+								      GDK_SELECTION_PRIMARY),
+					    galeon_tab_clipboard_text_received_cb, weak_ptr);
 	}
 	return FALSE;
 }
 
+static void
+galeon_tab_contextmenu_cb (GaleonEmbed *embed,
+		           GaleonEmbedEvent *event,
+			   GaleonTab *tab)
+{
+	galeon_tab_show_embed_popup (tab, event);
+}
+
 static void
 galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 			       GaleonTab *tab)
@@ -1068,6 +1367,24 @@ galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 	}
 }
 
+static void
+galeon_tab_popup_blocked_cb(GaleonEmbed *embed, const char *uri,  
+                            GaleonTab *tab)
+{
+        CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+
+	if (tab->priv->blocked_popup_uri)
+		g_free(tab->priv->blocked_popup_uri);
+
+	tab->priv->blocked_popup_uri = uri ? g_strdup(uri) : NULL;
+
+        if (tab->priv->is_active)
+        {
+                galeon_window_update_control(tab->priv->window,
+                                             StatusbarPopupBlockControl);
+        }
+}
+
 TabLoadStatus 
 galeon_tab_get_load_status (GaleonTab *tab)
 {
@@ -1124,12 +1441,34 @@ galeon_tab_get_location (GaleonTab *tab)
 	return tab->priv->location;
 }
 
+const char *
+galeon_tab_get_blocked_popup_uri(GaleonTab *tab)
+{
+	return tab->priv->blocked_popup_uri;
+}
+
+int
+galeon_tab_get_zoom(GaleonTab *tab)
+{
+	return tab->priv->zoom;
+}
+
+void
+galeon_tab_clear_blocked_popup_uri(GaleonTab *tab)
+{
+	if (tab->priv->blocked_popup_uri)
+	{
+		g_free(tab->priv->blocked_popup_uri);
+		tab->priv->blocked_popup_uri = NULL;
+	}
+}
+
 void 
 galeon_tab_set_location (GaleonTab *tab,
 			 char *location)
 {
 	if (tab->priv->location) g_free (tab->priv->location);
-	tab->priv->location = location;
+	tab->priv->location = g_strdup (location);
 }
 
 static void
@@ -1273,7 +1612,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	GaleonWindow *window;
 	GaleonEmbed  *embed;
 	
-	DEBUG_MSG (("gesture: %s\n", sequence));
+	LOG ("gesture: %s", sequence);
 	
 	if (!gestures_actions)
 	{
@@ -1287,7 +1626,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 		return;
 	}
 	
-	DEBUG_MSG (("action: %s\n", action));
+	LOG ("action: %s", action);
 	window = galeon_tab_get_window (tab);
 	embed = galeon_tab_get_embed (tab);
 	
@@ -1319,9 +1658,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "reload_bypass"))
 	{
-		galeon_embed_reload (embed, 
-				     EMBED_RELOAD_BYPASSCACHE |
-				     EMBED_RELOAD_BYPASSPROXY);
+		galeon_embed_reload (embed, EMBED_RELOAD_FORCE);
 	}
 	else if (!strcmp (action, "homepage"))
 	{
