@@ -15,6 +15,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "location-entry.h"
 #include "galeon-autocompletion-window.h"
@@ -22,19 +25,21 @@
 #include "gul-gobject-misc.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
+#include "galeon-debug.h"
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtkwindow.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkclipboard.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkseparatormenuitem.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkwindow.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-entry.h>
 #include <string.h>
 
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
-
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-
 /**
  * Private data
  */
@@ -58,6 +63,8 @@ struct _GaleonLocationEntryPrivate {
 #define AUTOCOMPLETION_DELAY 200
 #define SHOW_ALTERNATIVES_DELAY 500
 
+#define GALEON_LOCATION_ENTRY_HISTORY_ID	"galeon-url-history"
+
 /**
  * Private functions, only availble from this file
  */
@@ -75,6 +82,7 @@ static gint		galeon_location_entry_autocompletion_show_alternatives_to (GaleonLo
 static void		galeon_location_entry_autocompletion_window_url_activated_cb
 /***/								(GaleonAutocompletionWindow *aw, 
 								 const gchar *url, 
+								 gboolean new_tab_or_window,
 								 GaleonLocationEntry *w);
 static void		galeon_location_entry_list_event_after_cb (GtkWidget *list,
 								   GdkEvent *event, 
@@ -120,8 +128,8 @@ galeon_location_entry_class_init (GaleonLocationEntryClass *klass)
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
                 G_STRUCT_OFFSET (GaleonLocationEntryClass, galeon_location_entry_url_activated), 
 		NULL, NULL, 
-		galeon_marshal_VOID__STRING,
-		G_TYPE_NONE, 1, G_TYPE_STRING);
+		galeon_marshal_VOID__STRING_INT,
+		G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_BOOLEAN);
 }
 
 static gboolean
@@ -132,6 +140,39 @@ focus_out_event_cb (GtkWidget *entry, GdkEventFocus *event,
 	return FALSE;
 }
 
+static void
+entry_clear_activate_cb (GtkMenuItem *item, GtkEntry *entry)
+{
+	gtk_entry_set_text (entry, "");
+}
+
+static void
+entry_populate_popup_cb (GtkEntry *entry, GtkMenu *menu)
+{
+	GtkWidget *image;
+	GtkWidget *menuitem;
+
+	menuitem = gtk_separator_menu_item_new ();
+	gtk_widget_show (menuitem);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menuitem);
+
+	/* Clear and Copy mnemonics conflict, make custom menuitem */
+	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
+	/* To translators: the mnemonic shouldn't conflict with any of the
+	 * stock items in the GtkEntry context menu (Cut, Copy, Paste, Delete,
+	 * and Insert Unicode control character.) */
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("Cl_ear"));
+
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menuitem), image);
+
+	g_signal_connect (G_OBJECT(menuitem), "activate",
+			  G_CALLBACK(entry_clear_activate_cb), entry);
+
+	gtk_widget_show (image);
+	gtk_widget_show (menuitem);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menuitem);
+}
+
 static void 
 galeon_location_entry_init (GaleonLocationEntry *w)
 {
@@ -140,6 +181,9 @@ galeon_location_entry_init (GaleonLocationEntry *w)
 
 	galeon_location_entry_build (w);
 
+	g_signal_connect (w->priv->entry, "populate_popup",
+			  G_CALLBACK (entry_populate_popup_cb), NULL);
+
 	g_signal_connect (w->priv->entry, "focus_out_event",
 			  G_CALLBACK (focus_out_event_cb), w);
 }
@@ -162,7 +206,10 @@ galeon_location_entry_finalize_impl (GObject *o)
 		g_object_unref (G_OBJECT (p->autocompletion_window)); 
 	}
 
-	DEBUG_MSG (("GaleonLocationEntry finalized\n"));
+	g_free (p->last_completion);
+	g_free (p->before_completion);
+	g_free (p->autocompletion_key);
+	LOG ("GaleonLocationEntry finalized");
 
 	g_free (p);	
 	G_OBJECT_CLASS (gtk_hbox_class)->finalize (o);
@@ -174,13 +221,23 @@ galeon_location_entry_new (void)
 	return GALEON_LOCATION_ENTRY (g_object_new (GALEON_TYPE_LOCATION_ENTRY, NULL));
 }
 
+void
+galeon_location_entry_clear_history (void)
+{
+	GtkWidget *w;
+
+	w = gnome_entry_new (GALEON_LOCATION_ENTRY_HISTORY_ID);
+	gnome_entry_clear_history (GNOME_ENTRY (w));
+	gtk_widget_destroy (w);
+}
+
 static void
 galeon_location_entry_build (GaleonLocationEntry *w)
 {
 	GaleonLocationEntryPrivate *p = w->priv;
 	GtkWidget *list;
 
-	p->combo = gnome_entry_new ("galeon-url-history");
+	p->combo = gnome_entry_new (GALEON_LOCATION_ENTRY_HISTORY_ID);
 	p->entry = GTK_COMBO (p->combo)->entry;
 	gtk_widget_show (p->combo);
 	gtk_box_pack_start (GTK_BOX (w), p->combo, TRUE, TRUE, 0);
@@ -207,7 +264,7 @@ galeon_location_entry_autocompletion_show_alternatives_to (GaleonLocationEntry *
 	GaleonLocationEntryPrivate *p = w->priv;
 	if (p->autocompletion)
 	{
-		DEBUG_MSG (("+galeon_location_entry_autocompletion_show_alternatives_to\n"));
+		LOG ("+galeon_location_entry_autocompletion_show_alternatives_to");
 		galeon_location_entry_set_autocompletion_key (w);
 		galeon_location_entry_autocompletion_show_alternatives (w);
 	}
@@ -254,7 +311,7 @@ galeon_location_entry_autocompletion_to (GaleonLocationEntry *w)
 	gchar *text;
 	gchar *common_prefix;
 	
-	DEBUG_MSG (("in galeon_location_entry_autocompletion_to\n"));
+	LOG ("in galeon_location_entry_autocompletion_to");
 
 	galeon_location_entry_set_autocompletion_key (w);
 
@@ -270,8 +327,9 @@ galeon_location_entry_autocompletion_to (GaleonLocationEntry *w)
 		    || send != text_len)
 		{
 			/* the user is editing the entry, don't mess it */
-			DEBUG_MSG (("The user seems editing the text: pos = %d, strlen (text) = %d, sstart = %d, send = %d\n", 
-				    pos, strlen (text), sstart, send));
+			LOG ("The user seems editing the text: pos = %d, "
+			     "strlen (text) = %d, sstart = %d, send = %d", 
+			     pos, strlen (text), sstart, send);
 			p->autocompletion_timeout = 0;
 			return FALSE;
 		}		
@@ -279,7 +337,7 @@ galeon_location_entry_autocompletion_to (GaleonLocationEntry *w)
 
 	common_prefix = galeon_autocompletion_get_common_prefix (p->autocompletion);
 
-	DEBUG_MSG (("common_prefix: %s\n", common_prefix));
+	LOG ("common_prefix: %s", common_prefix);
 
 	if (common_prefix && (!p->before_completion || p->before_completion[0] == '\0'))
 	{
@@ -316,12 +374,41 @@ real_set_location (GaleonLocationEntry *w,
 		   const gchar *new_location)
 {
 	GaleonLocationEntryPrivate *p = w->priv;
-	int pos;
+	int pos = 0;
+	int start, end;
+	gchar* selection = NULL;
+
+	/* Get the current selection, it is removed when we do the delete_text 
+	 * below */
+	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (p->entry), &start, &end))
+	{
+		selection = gtk_editable_get_chars (GTK_EDITABLE (p->entry), start, end);
+	}
+
 	gtk_editable_delete_text (GTK_EDITABLE (p->entry), 0, -1);
-	gtk_editable_insert_text (GTK_EDITABLE (p->entry), new_location, g_utf8_strlen (new_location, -1),
-				  &pos);
+	gtk_editable_insert_text (GTK_EDITABLE (p->entry), new_location, 
+			          strlen (new_location), &pos);
+
+	/* Restore the selection into the PRIMARY clipboard as it will have just
+	 * been wiped */
+	if (selection)
+	{
+		gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+					selection, strlen (selection));
+		g_free (selection);
+	}
 }
 
+static void
+emit_activated_signal (GaleonLocationEntry *w, const char *url, gboolean new_tab_or_window)
+{
+	w->priv->editing = FALSE;
+
+	g_signal_emit (w, GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_URL_ACTIVATED], 
+                       0, url, new_tab_or_window);
+}
+
+
 /* this is from the old location entry, need to do the autocompletion before implementing this */
 static gboolean
 galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event, GaleonLocationEntry *w)
@@ -347,6 +434,25 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
                 suggest = FALSE;
 	}
 	
+
+	/* Handle any CTRL+Enter events */
+	if ( event->state & GDK_CONTROL_MASK && 
+	     ( event->keyval == GDK_KP_Enter || event->keyval == GDK_Return ) )
+	{
+		char *url = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
+
+		LOG ("In galeon_location_key_press_cb, going to %s", url);
+		   
+		galeon_location_entry_autocompletion_hide_alternatives (w);
+
+		emit_activated_signal (w, url, TRUE);
+
+		g_free (url);
+
+		return TRUE;
+	}
+
+	/* Ingore any CTRL+<key> commands (or backspace commands) */
         if ((event->state & GDK_CONTROL_MASK) ||
             (event->state == 0 && event->keyval == GDK_BackSpace))
         {
@@ -385,7 +491,7 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
         case GDK_Page_Up:
         case GDK_Page_Down:
 		galeon_location_entry_autocompletion_hide_alternatives (w);
-                //galeon_embed_grab_focus (window->active_embed);
+                /* galeon_embed_grab_focus (window->active_embed); */
                 return FALSE;
         case GDK_Tab:
         {
@@ -398,9 +504,11 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
                 text = galeon_location_entry_get_location (w);
 		galeon_location_entry_autocompletion_unselect_alternatives (w);
 
-		// TODO: heuristic completions like in galeon1 are missing
-                //common_prefix = 
-		//      auto_completion_complete_url_extended (text, suggest);
+		/*
+		 * TODO: heuristic completions like in galeon1 are missing
+		 * common_prefix = 
+		 *     auto_completion_complete_url_extended (text, suggest);
+		 */
 
 		if (p->autocompletion)
 		{
@@ -458,10 +566,13 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
                 }
                 else
                 {
-                        //real_set_location (w, embed->location);
+                        /* real_set_location (w, embed->location); */
                 }
                 break;
         default:
+		/* ignore keys that don't generate content (Shift, Ctrl, ...) */
+		if (event->length <= 0) break;
+
 		w->priv->editing = TRUE;
 		galeon_location_entry_autocompletion_unselect_alternatives (w);
 		if ((eel_gconf_get_boolean (CONF_COMPLETION_SHOW_LIST_AUTO))
@@ -485,14 +596,6 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
         return FALSE;
 }
 
-static void
-emit_activated_signal (GaleonLocationEntry *w, const char *url)
-{
-	w->priv->editing = FALSE;
-
-	g_signal_emit (w, GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_URL_ACTIVATED], 0, url);
-}
-
 static void
 galeon_location_entry_activate_cb (GtkEntry *entry, GaleonLocationEntry *w)
 {
@@ -502,9 +605,9 @@ galeon_location_entry_activate_cb (GtkEntry *entry, GaleonLocationEntry *w)
 
 	galeon_location_entry_autocompletion_hide_alternatives (w);
 
-	DEBUG_MSG (("In galeon_location_entry_activate_cb, going to %s\n", url));
+	LOG ("In galeon_location_entry_activate_cb, going to %s", url);
 
-	emit_activated_signal (w, url);
+	emit_activated_signal (w, url, FALSE);
 
 	g_free (url);
 }
@@ -515,7 +618,7 @@ galeon_location_entry_autocompletion_sources_changed_cb (GaleonAutocompletion *a
 {
 	GaleonLocationEntryPrivate *p = w->priv;
 
-	DEBUG_MSG (("in galeon_location_entry_autocompletion_sources_changed_cb\n"));
+	LOG ("in galeon_location_entry_autocompletion_sources_changed_cb");
 
         if (p->autocompletion_timeout == 0
 	    && p->last_completion
@@ -593,15 +696,17 @@ galeon_location_entry_set_autocompletion (GaleonLocationEntry *w,
 
 static void
 galeon_location_entry_autocompletion_window_url_activated_cb (GaleonAutocompletionWindow *aw, 
-							      const gchar *url, GaleonLocationEntry *w)
+							      const gchar *url, 
+							      gboolean new_tab_or_window,
+							      GaleonLocationEntry *w)
 {
 	real_set_location (w, url);
 
-	DEBUG_MSG (("In location_entry_autocompletion_window_url_activated_cb, going to %s\n", url));
+	LOG ("In location_entry_autocompletion_window_url_activated_cb, going to %s", url);
 
 	galeon_location_entry_autocompletion_hide_alternatives (w);
 
-	emit_activated_signal (w, url);
+	emit_activated_signal (w, url, new_tab_or_window);
 }
 
 static void
@@ -610,7 +715,7 @@ galeon_location_entry_autocompletion_window_hidden_cb (GaleonAutocompletionWindo
 {
 	GaleonLocationEntryPrivate *p = w->priv;
 
-	DEBUG_MSG (("In location_entry_autocompletion_window_hidden_cb\n"));
+	LOG ("In location_entry_autocompletion_window_hidden_cb");
 
 	p->autocompletion_window_visible = FALSE;
 
@@ -660,7 +765,7 @@ galeon_location_entry_editable_changed_cb (GtkEditable *editable, GaleonLocation
 		if (url && url[0] != '\0')
 		{
 			p->hack_for_going_to_the_site_when_clicking_in_the_location_history = FALSE;
-			emit_activated_signal (e, url);
+			emit_activated_signal (e, url, FALSE);
 			g_free (url);
 		}
 	}
